@@ -1,4 +1,4 @@
-# Copyright (C) 2013 - 2019  Metrum Research Group
+# Copyright (C) 2013 - 2021 Metrum Research Group
 #
 # This file is part of mrgsolve.
 #
@@ -25,56 +25,397 @@ get_length <- function(what) sum(sapply(what,length))
 # GENERIC ----------------------------------------------------------------------
 handle_spec_block <- function(x,...) UseMethod("handle_spec_block")
 
+# DEFAULT ------
 #' @export
-handle_spec_block.default <- function(x,...) {
+handle_spec_block.default <- function(x, ...) {
   return(dump_opts(x))
 }
 
+#' Functions to parse code blocks
+#' 
+#' Most of the basic blocks are listed in this help topic.  
+#' But see also [PKMODEL()] which has more-involved 
+#' options and is documented separately.
+#' 
+#' @param x data
+#' @param env parse environment
+#' @param pos block position
+#' @param annotated logical
+#' @param object the name of an object in `ENV`
+#' @param as_object indicates that object code is being provided
+#' @param covariates logical
+#' @param name block name
+#' @param labels aliases to use for simulated ETA values
+#' @param prefix a prefix to add to the label
+#' @param type internal use
+#' @param oclass internal use
+#' @param unlinked internal use
+#' @param fill deprecated; not used
+#' @param ... passed
+#' 
+#' @details
+#' 
+#' When using `object` or `as_object` populate the block contents, the following
+#' types are required
+#' 
+#' - `PARAM`: a named list
+#' - `INIT` : a named list
+#' - `THETA` : a numeric vector; names are ignored
+#' - `CMT`: a character vector
+#' - `OMEGA`: matrix; set rownames on the matrix to create ETA labels; setting 
+#'   rownames is the only way to specify `labels` when working through the 
+#'   `object` or `as_object` directives
+#' - `SIGMA`: matrix; set rownames on the matrix to create EPS labels; setting 
+#'   rownames is the only way to specify `labels` when working through the 
+#'   `object` or `as_object` directives
+#' 
+#' @name BLOCK_PARSE
+#' @rdname BLOCK_PARSE
+#' 
+#' @md
+#' @seealso [PKMODEL()]
+NULL
+
+# PARAM / FIXED ----------------------------------------------------------------
+
+#' @export
+handle_spec_block.specPARAM <- function(x, ...) {
+  scrape_and_call(x, pass = "PARAM", ...)
+}
+
+#' @rdname BLOCK_PARSE
+PARAM <- function(x, 
+                  env, 
+                  pos = 1, 
+                  annotated = FALSE,
+                  object = NULL, 
+                  as_object = FALSE,
+                  covariates = FALSE, ...) {
+  
+  if(is.character(object)) {
+    if(isTRUE(as_object)) {
+      wstop("cannot have both @object and @as_object in a block")  
+    }
+    x <- object
+    as_object <- TRUE
+    envir <- env$ENV
+  } else {
+    check_block_data(x, env, pos)
+    envir <- list()
+  }
+  
+  if(as_object) {
+    x <- evaluate_at_code(
+      x, c("list", "parameter_list"), 
+      "PARAM", 
+      pos, 
+      envir, 
+      named = TRUE
+    )
+    env[["param"]][[pos]] <- x
+    return(NULL)
+  }
+  
+  if(annotated) {
+    context <- env[["incoming_names"]][pos]
+    context <- as.character(glue("parse annotated parameter block ({context})"))
+    l <- parse_annot(x, block = "PARAM", envir = env$ENV, context = context)
+    env[["param"]][[pos]] <- l[["v"]]
+    env[["annot"]][[pos]] <- l[["an"]]
+  } else {
+    x <- tolist(x,envir=env$ENV) 
+    if(length(x) > 0 & !is_named(x)) {
+      msg <- " invalid model specification
+       Block no: {pos}
+       Block type: {env$incoming_names[pos]}
+       Some parameters are missing names
+      "
+      stop(glue(msg), call. = FALSE)
+    }
+    env[["param"]][[pos]] <- x
+  }
+  
+  if(covariates) {
+    env[["covariates"]] <- c(
+      env[["covariates"]], names(env[["param"]][[pos]])
+    )
+  }
+  
+  return(NULL)
+}
+
+#' @export
+handle_spec_block.specFIXED <- function(x, ...) {
+  scrape_and_call(x, pass = "FIXED", ...)
+}
+
+#' @rdname BLOCK_PARSE
+FIXED <- function(x, env, pos = 1, annotated = FALSE, ...) {
+  
+  check_block_data(x, env, pos)
+  
+  if(annotated) {
+    context <- env[["incoming_names"]][pos]
+    context <- glue("parse annotated fixed block ({context})")
+    l <- parse_annot(x, block= "FIXED", envir = env$ENV, context = context)
+    env[["fixed"]][[pos]] <- l[["v"]]
+    env[["annot"]][[pos]] <- l[["an"]]
+  } else {
+    x <- tolist(x,envir=env$ENV) 
+    env[["fixed"]][[pos]] <- x
+  }
+  
+  return(NULL)
+}
+
+# THETA ------------------------------------------------------------------------
+
+#' @export
+handle_spec_block.specTHETA <- function(x, ...) {
+  scrape_and_call(x, pass = "THETA", narrow = FALSE, ...)
+}
+
+#' @rdname BLOCK_PARSE
+THETA <- function(x, 
+                  env, 
+                  pos = 1, 
+                  annotated = FALSE, 
+                  object = NULL, 
+                  as_object = FALSE, 
+                  name = "THETA", 
+                  fill = NULL, ...) {
+  
+  if(!is.null(fill)) {
+    x <- fill
+    as_object <- TRUE
+  }
+  
+  if(is.character(object)) {
+    if(isTRUE(as_object)) {
+      wstop("cannot have both @object and @as_object in a block")  
+    }
+    x <- object  
+    as_object <- TRUE
+    envir <- env$ENV
+  } else {
+    check_block_data(x, env, pos)
+    envir <- list()
+  }
+  
+  if(isTRUE(as_object)) {
+    x <- evaluate_at_code(x, c("numeric", "integer"), "THETA", pos, envir)
+  }
+  
+  if(annotated) {
+    l <- parse_annot(x, noname = TRUE, block = "THETA", envir = env$ENV)
+    x <- as.numeric(l[["v"]])
+  } else {
+    x <- tolist(paste0(cvec_cs(x),collapse=','), envir = env$ENV)
+  }
+  
+  x <- x[!is.na(x)]
+  
+  if(length(x)==0) name <- character(0)
+  
+  names(x) <- paste0(name, seq_along(x))
+  
+  env[["param"]][[pos]] <- x
+  
+  return(NULL)
+}
+
+# INIT /CMT --------------------------------------------------------------------
+
+#' @export
+handle_spec_block.specINIT <- function(x, ...) {
+  scrape_and_call(x, pass="INIT", ...)
+}
+
+#' @rdname BLOCK_PARSE
+INIT <- function(x, 
+                 env, 
+                 pos = 1, 
+                 annotated = FALSE, 
+                 object = NULL, 
+                 as_object = FALSE, ...) {
+  
+  if(is.character(object)) {
+    if(isTRUE(as_object)) {
+      wstop("cannot have both @object and @as_object in a block")  
+    }
+    x <- object
+    as_object <- TRUE
+    envir <- env$ENV
+  } else {
+    check_block_data(x, env, pos)  
+    envir <- list()
+  }
+  
+  if(as_object) {
+    x <- evaluate_at_code(x, "list", "INIT", pos, envir, named = TRUE)
+    env[["init"]][[pos]] <- x
+    return(NULL)
+  }
+  
+  if(annotated) {
+    context <- env[["incoming_names"]][pos]
+    context <- glue("parse annotated init block ({context})")
+    l <- parse_annot(x, block = "INIT", envir = env$ENV, context = context)
+    env[["init"]][[pos]] <- l[["v"]]
+    env[["annot"]][[pos]] <- l[["an"]]
+  } else {
+    x <- tolist(x,envir=env$ENV) 
+    if(length(x) > 0 & !is_named(x)) {
+      msg <- " invalid model specification
+       Block no: {pos}
+       Block type: {env$incoming_names[pos]}
+       Some initials are missing names
+      "
+      stop(glue(msg), call. = FALSE)
+    }
+    env[["init"]][[pos]] <- x
+  }
+  return(NULL)
+}
+
+#' @export
+handle_spec_block.specCMT <- function(x, ...) {
+  scrape_and_call(x, pass = "CMT", narrow = FALSE, ...)
+}
+
+#' @rdname BLOCK_PARSE
+CMT <- function(x, 
+                env, 
+                pos = 1, 
+                annotated = FALSE, 
+                object = NULL, 
+                as_object = FALSE, ...) {
+  
+  if(is.character(object)) {
+    if(isTRUE(as_object)) {
+      wstop("cannot have both @object and @as_object in a block")  
+    }
+    x <- object 
+    as_object <- TRUE
+    envir <- env$ENV
+  } else {
+    check_block_data(x, env, pos)
+    envir <- list()
+  }
+  
+  if(as_object) {
+    x <- evaluate_at_code(x, "character", "CMT", pos, envir)
+    x <- setNames(rep(0, length(x)), x)
+    env[["init"]][[pos]] <- x
+    return(NULL)
+  }
+  if(annotated) {
+    context <- env[["incoming_names"]][pos]
+    context <- glue("parse annotated compartment block ({context})")
+    l <- parse_annot(
+      x, novalue = TRUE, block = "CMT", envir = env$ENV, 
+      context = context
+    )
+    env[["annot"]][[pos]] <- l[["an"]]
+    x <- names(l[["v"]])
+  } else {
+    x <- cvec_cs(x)
+  }
+  
+  l <- rep(0,length(x))
+  names(l) <- x
+  env[["init"]][[pos]] <- as.list(l)
+  return(NULL)
+}
+
+#' @export
+handle_spec_block.specVCMT <- handle_spec_block.specCMT
+
+#' @export
+handle_spec_block.specCMTN <- function(x, ...) {
+  cvec_cs(dump_opts(x))
+}
+
+# CAPTURE ----------------------------------------------------------------------
+
+#' @export
+handle_spec_block.specCAPTURE <- function(x, ...) {
+  scrape_and_call(x, pass = "CAPTURE", narrow = TRUE, ...)
+}
+
+#' @rdname BLOCK_PARSE
+CAPTURE <- function(x, env, pos = 1, annotated = FALSE, ...) {
+  
+  if(annotated) {
+    context <- env[["incoming_names"]][pos]
+    context <- glue("parse annotated capture block ({context})")
+    l <- parse_annot(
+      x, novalue = TRUE, block = "CAPTURE", envir = env$ENV, 
+      context = context
+    )
+    env[["annot"]][[pos]] <- l[["an"]]
+    x <- names(l[["v"]])
+  } else {
+    x <- cvec_cs(x)
+  }
+  
+  check_block_data(x, env, pos)
+  
+  env[["capture"]][[pos]] <- x
+  
+  return(NULL)
+}
+
 # OMEGA /SIGMA -----------------------------------------------------------------
 
 #' @export
-handle_spec_block.specOMEGA <- function(x,...) {
+handle_spec_block.specOMEGA <- function(x, ...) {
   scrape_and_call(
     x,
-    pass="specMATRIX",
-    def=list(oclass="omegalist",type="omega"),
-    narrow=FALSE,...
+    pass="HANDLEMATRIX",
+    def=list(oclass = "omegalist", type = "omega"),
+    narrow = FALSE, ...
   )
 }
 
 #' @export
-handle_spec_block.specSIGMA <- function(x,...) {
+handle_spec_block.specSIGMA <- function(x, ...) {
   scrape_and_call(
     x,
-    pass="specMATRIX",
-    def=list(oclass="sigmalist",type="sigma"),
-    narrow=FALSE,...
+    pass="HANDLEMATRIX",
+    def=list(oclass = "sigmalist", type = "sigma"),
+    narrow = FALSE, ...
   )
 }
 
-specMATRIX <- function(x,
-                       oclass,type, annotated = FALSE,
-                       env, pos=1, as_object = FALSE,
-                       name="...", prefix="", labels=NULL,
-                       object=NULL, unlinked=FALSE,...) {
+#' @rdname BLOCK_PARSE
+HANDLEMATRIX <- function(x,
+                         env,
+                         pos = 1, 
+                         annotated = FALSE,
+                         object = NULL, 
+                         as_object = FALSE,
+                         name = "...",
+                         type = NULL,
+                         oclass = "", 
+                         prefix = "", 
+                         labels = NULL,
+                         unlinked = FALSE, ...) {
   
-  if(is.null(object)) check_block_data(x,env$ENV,pos)
-  
-  if(as_object) {
-    expect <- paste0(type, "list")
-    expect <- c("matrix",expect)
-    if(type=="omega") {
-      fun <- omat  
-    } else {
-      fun <- smat  
+  if(is.character(object)) {
+    if(isTRUE(as_object)) {
+      wstop("cannot have both @object and @as_object in a block")  
     }
-    x <- evaluate_at_code(x, expect, toupper(type), pos, env, fun)
-    env[[type]][[pos]] <- x
-    return(NULL)
+    x <- object
+    as_object <- TRUE
+    envir <- env$ENV
+  } else {
+    check_block_data(x, env, pos)
+    envir <- list()
   }
   
   anl <- grepl(":",x,fixed=TRUE)
-  if(annotated) {
+  if(annotated & !as_object) {
     types <- charcount(x[anl],":")
     if(all(types==1)) {
       unlinked <- TRUE 
@@ -92,16 +433,17 @@ specMATRIX <- function(x,
     
     l <- parse_annot(
       x[anl],
-      name_value=FALSE,
-      block=toupper(type),
-      envir=env$ENV,novalue=novalue
+      name_value = FALSE,
+      block = toupper(type),
+      envir = env$ENV, 
+      novalue = novalue
     )
     
     if(unlinked) {
       l[["v"]] <- as.numeric(cvec_cs(x[!anl])) 
     }
     
-    d <- modMATRIX(l[["v"]],context=oclass,...)
+    d <- modMATRIX(l[["v"]], context = oclass, ...)
     labels <- l[["an"]][["name"]]
     env[["annot"]][[pos]] <- l[["an"]]
     
@@ -113,11 +455,17 @@ specMATRIX <- function(x,
     }
     
   } else {
-    if(any(anl)) x <- x[!anl]
-    if(is.null(object)) {
-      d <- modMATRIX(x,context=oclass,...) 
+    
+    if(isTRUE(as_object)) {
+      expect <- paste0(type, "list")
+      expect <- c("matrix", expect)
+      d <- evaluate_at_code(x, expect, toupper(type), pos, envir)
+      if(is.null(labels)) {
+        labels <- rownames(d)  
+      }
     } else {
-      d <- get(object,env$ENV)
+      if(any(anl)) x <- x[!anl]
+      d <- modMATRIX(x, context = oclass, ...)
     }
   }
   
@@ -126,12 +474,12 @@ specMATRIX <- function(x,
   if(is.null(labels)) {
     labels <- rep(".", nrow(d))
   } else {
-    labels <- paste0(prefix,cvec_cs(labels))
+    labels <- paste0(prefix, cvec_cs(labels))
   }
   
-  d <- setNames(list(d),name)
+  d <- setNames(list(d), name)
   
-  x <- create_matlist(d,class=oclass,labels=list(labels))
+  x <- create_matlist(d, class = oclass, labels = list(labels))
   
   env[[type]][[pos]] <- x
   
@@ -141,13 +489,13 @@ specMATRIX <- function(x,
 # specTABLE --------------------------------------------------------------------
 
 #' @export 
-handle_spec_block.specTABLE <- function(x,env,...) {
+handle_spec_block.specTABLE <- function(x, env, ...) {
   
   x <- dump_opts(x)
   
   pos <- attr(x,"pos")
   
-  check_block_data(x,env$ENV,pos)
+  check_block_data(x, env, pos)
   
   if(any(grepl("table(", x,fixed=TRUE))) {
     stop("The table(name) = value; macro has been deprecated.\n",  
@@ -158,269 +506,54 @@ handle_spec_block.specTABLE <- function(x,env,...) {
   return(x)
 }
 
-#' Functions to parse code blocks
-#' 
-#' Most of the basic blocks are listed in this help topic.  
-#' But see also \code{\link{PKMODEL}} which has more-involved 
-#' options and is documented separately.
-#' 
-#' @param x data
-#' @param env parse environment
-#' @param annotated logical
-#' @param covariates logical
-#' @param name block name
-#' @param pos block position
-#' @param fill data to use for block contents
-#' @param as_object indicates that object code is being provided
-#' @param ... passed
-#' 
-#' @name BLOCK_PARSE
-#' @rdname BLOCK_PARSE
-#' 
-#' @seealso \code{\link{PKMODEL}}
-NULL
-
-# PARAM / FIXED ----------------------------------------------------------------
-
-#' @export
-handle_spec_block.specPARAM <- function(x,...) {
-  scrape_and_call(x,pass="PARAM",...)
-}
-
-#' @rdname BLOCK_PARSE
-PARAM <- function(x,env,annotated=FALSE,covariates=FALSE,pos=1,as_object=FALSE,...) {
-  
-  check_block_data(x,env$ENV,pos)
-  
-  if(as_object) {
-    x <- evaluate_at_code(x, c("list", "parameter_list"), "PARAM", pos, env)
-    env[["param"]][[pos]] <- x
-    return(NULL)
-  }
-  
-  if(annotated) {
-    context <- glue("parse annotated parameter block ({pos})")
-    l <- parse_annot(x,block="PARAM",envir=env$ENV,context = context)
-    env[["param"]][[pos]] <- l[["v"]]
-    env[["annot"]][[pos]] <- l[["an"]]
-  } else {
-    x <- tolist(x,envir=env$ENV) 
-    env[["param"]][[pos]] <- x
-  }
-  
-  if(covariates) {
-    env[["covariates"]] <- c(
-      env[["covariates"]], names(env[["param"]][[pos]])
-    )
-  }
-  
-  return(NULL)
-}
-
-#' @export
-handle_spec_block.specFIXED <- function(x,...) {
-  scrape_and_call(x,pass="FIXED",...)
-}
-
-#' @rdname BLOCK_PARSE
-FIXED <- function(x,env,annotated=FALSE,pos=1,...) {
-  
-  check_block_data(x,env$ENV,pos)
-  
-  if(annotated) {
-    context <- glue("parse annotated fixed block ({pos})")
-    l <- parse_annot(x,block="FIXED",envir=env$ENV,context=context)
-    env[["fixed"]][[pos]] <- l[["v"]]
-    env[["annot"]][[pos]] <- l[["an"]]
-  } else {
-    x <- tolist(x,envir=env$ENV) 
-    env[["fixed"]][[pos]] <- x
-  }
-  
-  return(NULL)
-}
-
-# THETA ------------------------------------------------------------------------
-
-#' @export
-handle_spec_block.specTHETA <- function(x,...) {
-  scrape_and_call(x,pass="THETA",narrow=FALSE,...)
-}
-
-#' @rdname BLOCK_PARSE
-THETA <- function(x, env, annotated=FALSE, pos=1, name="THETA", fill = NULL,
-                  ...) {
-  if(!is.null(fill)) {
-    x <- eval(parse(text = fill))   
-  }
-  
-  check_block_data(x,env$ENV,pos)
-  
-  if(annotated) {
-    l <- parse_annot(x,noname=TRUE,block="THETA",envir=env$ENV)
-    x <- as.numeric(l[["v"]])
-  } else {
-    x <- tolist(paste0(cvec_cs(x),collapse=','),envir=env$ENV)
-  }
-  
-  x <- x[!is.na(x)]
-  
-  if(length(x)==0) name <- character(0)
-  
-  names(x) <- paste0(name, seq_along(x))
-  
-  env[["param"]][[pos]] <- x
-  
-  return(NULL)
-}
-
-# INIT /CMT --------------------------------------------------------------------
-
-#' @export
-handle_spec_block.specINIT <- function(x,...) {
-  scrape_and_call(x,pass="INIT",...)
-}
-
-#' @rdname BLOCK_PARSE
-INIT <- function(x,env,annotated=FALSE,pos=1,as_object=FALSE,...) {
-  
-  check_block_data(x,env$ENV,pos)
-  
-  if(as_object) {
-    x <- evaluate_at_code(x, "list", "INIT", pos, env, as.list)
-    env[["init"]][[pos]] <- x
-    return(NULL)
-  }
-  
-  if(annotated) {
-    context <- glue("parse annotated init block ({pos})")
-    l <- parse_annot(x,block="INIT",envir=env$ENV,context=context)
-    env[["init"]][[pos]] <- l[["v"]]
-    env[["annot"]][[pos]] <- l[["an"]]
-  } else {
-    x <- tolist(x,envir=env$ENV) 
-    env[["init"]][[pos]] <- x
-  }
-  return(NULL)
-}
-
-#' @export
-handle_spec_block.specCMT <- function(x,...) {
-  scrape_and_call(x,pass="CMT",narrow=FALSE,...)
-}
-
-#' @rdname BLOCK_PARSE
-CMT <- function(x,env,annotated=FALSE,pos=1, as_object = FALSE, ...) {
-  check_block_data(x,env$ENV,pos)
-  if(as_object) {
-    x <- evaluate_at_code(x, "character", "CMT", pos, env, as.character)
-    x <- setNames(rep(0,length(x)), x)
-    env[["init"]][[pos]] <- x
-    return(NULL)
-  }
-  if(annotated) {
-    context <- glue("parse annotated compartment block ({pos})")
-    l <- parse_annot(x,novalue=TRUE,block="CMT",envir=env$ENV,context=context)
-    env[["annot"]][[pos]] <- l[["an"]]
-    x <- names(l[["v"]])
-  } else {
-    x <- cvec_cs(x)
-  }
-  
-  l <- rep(0,length(x))
-  names(l) <- x
-  env[["init"]][[pos]] <- as.list(l)
-  return(NULL)
-}
-
-#' @export
-handle_spec_block.specVCMT <- handle_spec_block.specCMT
-
-
-#' @export
-handle_spec_block.specCMTN <- function(x,...) {
-  cvec_cs(dump_opts(x))
-}
-
-# SET --------------------------------------------------------------------------
-
-#' @export
-handle_spec_block.specSET <- function(x,...) {
-  tolist(dump_opts(x))
-}
 
 # NMXML --------------------------------
 #' @export
-handle_spec_block.specNMXML <- function(x,...) {
-  parseNMXML(dump_opts(x),...)
-}
-
-## Functions for handling code blocks
-parseNMXML <- function(x,env,...) {
-  pos <- attr(x,"pos")
-  x <- tolist(x,envir=env$ENV)
-  xml <- do.call(nmxml,x)
+handle_spec_block.specNMXML <- function(x, env, ...) {
+  x <- dump_opts(x)
+  pos <- attr(x, "pos")
+  x <- tolist(x, envir = env$ENV)
+  x$env <- env
+  xml <- do.call(nmxml, x)
   env[["param"]][[pos]] <- xml$theta
   env[["omega"]][[pos]] <- xml$omega
   env[["sigma"]][[pos]] <- xml$sigma
+  env[["nm_import"]] <- c(env[["nm_import"]], xml[["file"]])
   return(NULL)
 }
 
 # NMEXT --------------------------------
 #' @export
-handle_spec_block.specNMEXT <- function(x,env,...) {
+handle_spec_block.specNMEXT <- function(x, env, ...) {
   x <- dump_opts(x)
-  pos <- attr(x,"pos")
-  x <- tolist(x,envir=env$ENV)
-  ext <- do.call(nmext,x)
+  pos <- attr(x, "pos")
+  x <- tolist(x, envir = env$ENV)
+  x$env <- env
+  ext <- do.call(nmext, x)
   env[["param"]][[pos]] <- ext$theta
   env[["omega"]][[pos]] <- ext$omega
   env[["sigma"]][[pos]] <- ext$sigma
+  env[["nm_import"]] <- c(env[["nm_import"]], ext[["file"]])
   return(NULL)
 }
 
-# CAPTURE ----------------------------------------------------------------------
-
-#' @export
-handle_spec_block.specCAPTURE <- function(x,...) {
-  scrape_and_call(x,pass="CAPTURE",narrow=TRUE,...)
-}
-
-#' @rdname BLOCK_PARSE
-CAPTURE <- function(x,env,annotated=FALSE,pos=1,...) {
-  
-  if(annotated) {
-    context <- glue("parse annotated capture block ({pos})")
-    l <- parse_annot(x,novalue=TRUE,block="CAPTURE",envir=env$ENV,context=context)
-    env[["annot"]][[pos]] <- l[["an"]]
-    x <- names(l[["v"]])
-  } else {
-    x <- cvec_cs(x)
-  }
-  
-  check_block_data(x,env$ENV,pos)
-  
-  env[["capture"]][[pos]] <- x
-  
-  return(NULL)
-}
 
 # PRED -------------------------------------------------------------------------
 
 #' @export
-handle_spec_block.specPRED <- function(x,env,...) {
+handle_spec_block.specPRED <- function(x, env, ...) {
   x <- scrape_opts(x)
   x$env <- env
-  x$pos <- attr(x,"pos")
-  do.call("PRED",x)
+  x$pos <- attr(x, "pos")
+  do.call("PRED", x)
 }
 
-PRED <- function(x,env,...) {
+PRED <- function(x, env, ...) {
   if(any("MAIN"==env[["blocks"]])) {
-    stop("$MAIN not allowed when $PRED is used",call.=FALSE)  
+    stop("$MAIN not allowed when $PRED is used", call.=FALSE)  
   }
   if(any("TABLE"==env[["blocks"]])) {
-    stop("$TABLE not allowed when $PRED is used",call.=FALSE)  
+    stop("$TABLE not allowed when $PRED is used", call.=FALSE)  
   }
   if(any("PKMODEL"==env[["blocks"]])) {
     stop("$PKMODEL not allowed when $PRED is used",call.=FALSE)  
@@ -440,19 +573,19 @@ PRED <- function(x,env,...) {
 # INCLUDE ----------------------------------------------------------------------
 
 #' @export
-handle_spec_block.specINCLUDE <- function(x,env,...) { 
+handle_spec_block.specINCLUDE <- function(x, env, ...) { 
   
   x <- cvec_c_tr(dump_opts(x))
   
   pos <- attr(x, "pos")
   
-  check_block_data(x,env$ENV,pos)
+  check_block_data(x, env, pos)
   
-  if(any(grepl("[\"\']",x,perl=TRUE))) {
+  if(any(grepl("[\"\']", x, perl = TRUE))) {
     stop("Items in $INCLUDE should not contain quotation marks.",call.=FALSE) 
   }
   
-  if(any(!grepl("^.*\\.h$",x,perl=TRUE))) {
+  if(any(!grepl("^.*\\.h$", x, perl = TRUE))) {
     warning(
       "$INCLUDE expects file names ending with '.h'",
       call.=FALSE,
@@ -463,7 +596,7 @@ handle_spec_block.specINCLUDE <- function(x,env,...) {
   x <- file.path(env[["project"]],x)
   
   if(!all(file_exists(x))) {
-    message("Attempting to include:\n",paste0(" ",x, collapse = "\n"))
+    message("Attempting to include:\n", paste0(" ", x, collapse = "\n"))
     stop(
       "All header files in $INCLUDE must exist in the project directory.",
       call.=FALSE
@@ -482,13 +615,13 @@ form_includes <- function(files) {
 # PLUGIN -----------------------------------------------------------------------
 
 #' @export
-handle_spec_block.specPLUGIN <- function(x,env,...) {
+handle_spec_block.specPLUGIN <- function(x, env, ...) {
   
   x <- unique(cvec_c_tr(x))
   
   pos <- attr(x,"pos")
   
-  check_block_data(x,env$ENV,pos)
+  check_block_data(x, env, pos)
   
   if(length(x) ==0) return(list())
   
@@ -498,7 +631,7 @@ handle_spec_block.specPLUGIN <- function(x,env,...) {
 # TRANSIT ----------------------------------------------------------------------
 # nocov start
 #' @export
-handle_spec_block.specTRANSIT <- function(x,env,...) {
+handle_spec_block.specTRANSIT <- function(x, env, ...) {
   x <- scrape_opts(x, narrow=FALSE)
   x$env <- env
   x$pos <-  attr(x,"pos")
@@ -534,7 +667,7 @@ TRANSIT <- function(x,env, n, tr, name = "TRANSIT", pos=1) {
 # PKMODEL ----------------------------------------------------------------------
 
 #' @export
-handle_spec_block.specPKMODEL <- function(x,env,...) {
+handle_spec_block.specPKMODEL <- function(x, env, ...) {
   pos <- attr(x,"pos")
   x <- scrape_opts(x, narrow=FALSE)
   x$env <- env
@@ -586,8 +719,9 @@ handle_spec_block.specPKMODEL <- function(x,env,...) {
 #' }
 #' 
 #' @seealso \code{\link{BLOCK_PARSE}}
-PKMODEL <- function(ncmt=1,depot=FALSE,cmt=NULL, trans=pick_trans(ncmt,depot),
-                    env=list(), pos=1,...) {
+PKMODEL <- function(ncmt = 1, depot = FALSE, cmt = NULL, 
+                    trans = pick_trans(ncmt, depot), env = list(), 
+                    pos = 1, ...) {
   if(is.character(cmt)) {
     cmt <- cvec_cs(cmt)
     ncmt <- length(cmt)
@@ -598,19 +732,19 @@ PKMODEL <- function(ncmt=1,depot=FALSE,cmt=NULL, trans=pick_trans(ncmt,depot),
   }
   stopifnot(ncmt %in% c(1,2))
   advan <- pick_advan(ncmt,depot)
-
+  
   return(list(advan=advan, trans=trans, n=ncmt))
 }
 
 ## Collect PKMODEL information; hopefully will be deprecating ADVAN2 and ADVAN4 soon
-collect_subr <- function(x,what=c("PKMODEL")) {
+collect_subr <- function(x, what = "PKMODEL") {
   
   if("PRED" %in% names(x)) {
     ans <- list(advan = 0, trans = 0, n = 0)
     return(ans)
   }
   
-  ans <- list(advan=13,trans=1)
+  ans <- list(advan = 13, trans = 1)
   
   y <- x[names(x) %in% what]
   
@@ -634,7 +768,7 @@ collect_subr <- function(x,what=c("PKMODEL")) {
   return(ans)
 }
 
-dosing_cmts <- function(x,what) {
+dosing_cmts <- function(x, what) {
   if(!is.character(x)) return(character(0))
   x <- unlist(strsplit(x,"\n",fixed=TRUE),use.names=FALSE)
   m <- regexpr("(ALAG|F|R|D)\\_[^= ]+", x, perl=TRUE)
@@ -680,36 +814,39 @@ check_pred_symbols <- function(x,code) {
 # YAML -------------------------------------------------------------------------
 # nocov start
 #' @export
-handle_spec_block.specYAML <- function(x,env,...) {
+handle_spec_block.specYAML <- function(x, env, ...) {
   
-  if(!requireNamespace("yaml")) {
-    stop("the yaml package must be installed to process YAML blocks.",
-         call.=FALSE)
+  if(!requireNamespace("yaml", quietly = TRUE)) {
+    stop(
+      "the yaml package must be installed to process YAML blocks.",
+      call.=FALSE
+    )
   }
-  
   pos <- attr(x,"pos")
   x <- paste0(x, collapse = "\n")  
-  x <- yaml::yaml.load(x, eval.expr=TRUE)
-  
-  annotated3 <- function(value=0, descr = '.', unit = '.') {
+  x <- yaml::yaml.load(x, eval.expr = TRUE)
+  annotated3 <- function(value = 0, descr = '.', unit = '.') {
     tibble(value = value, descr = descr, unit = unit)
   }
   annotated2 <- function(descr = '.', unit = '.') {
-    annotated3(value = NA_real_,descr,unit)
+    annotated3(value = NA_real_, descr, unit)
   }
-  handle_annotated <- function(data,what,block) {
-    x <- lapply(data,as.list)
-    x <- lapply(x,FUN=do.call,what = what) %>% bind_rows()
-    mutate(x, block=block, name = names(data))
+  handle_annotated <- function(data, what, block) {
+    x <- lapply(data, as.list)
+    x <- lapply(x, FUN = do.call, what = what) 
+    x <- bind_rows(x)
+    label <-  names(data) 
+    if(is.null(labels) & is.character(data)) label <- data
+    mutate(x, block = block, name = label)
   }
-  
   names(x) <- tolower(names(x))
   pars <- handle_annotated(x$param, annotated3, "PARAM")
   cmt <- handle_annotated(x$cmt, annotated2,"CMT")
   cmt <- mutate(cmt, value = 0)
   init <- handle_annotated(x$init, annotated3, "INIT")
   out <- handle_annotated(x$capture, annotated2, "CAPTURE")
-  ann <- bind_rows(pars,cmt,init,out) %>% select(c("block","name","descr","value","unit"))
+  ann <- bind_rows(pars,cmt,init,out)
+  ann <- select(ann, c("block","name","descr","unit"))
   parameters <- setNames(as.list(pars[["value"]]),pars[["name"]])
   compartments <- setNames(as.list(cmt[["value"]]),cmt[["name"]])
   initials <- setNames(as.list(init[["value"]]),init[["name"]])
@@ -749,7 +886,9 @@ handle_spec_block.specODE <- function(x,env,...) {
   re <- "\\bETA\\([0-9]+\\)"
   chk <- grepl(re,x)
   if(any(chk)) {
-    er1 <- "ETA(n) is not allowed in ODE block:"
+    ode <- env$incoming_names[pos]
+    er1 <- "ETA(n) is not allowed in {ode} block:"
+    er1 <- as.character(glue(er1))
     er2 <- paste0("\n * ", x[which(chk)])
     stop(er1,er2,call.=FALSE)
   }
@@ -770,7 +909,7 @@ handle_spec_block.specBLOCK <- function(x,env,...) {
   do.call("BLOCK",x)
 }
 
-BLOCK <- function(x,env,type,code=NULL,get=NULL,...) {
+BLOCK <- function(x, env, type, code = NULL, get = NULL, ...) {
   x <- structure(.Data=code, class=paste0("spec",type),pos=1)
   handle_spec_block(x,env)
 }
