@@ -183,21 +183,21 @@ as.cvec <- function(x) {
 }
 
 # collapse a character vector back to length n (undo strsplit)
-collapsen <- function(string,pattern,n=3) {
+collapsen <- function(string,collapse,n=3) {
     if(length(string) <= n) return(string)
-    if(n==1) return(paste0(string, collapse = pattern))
+    if(n==1) return(paste0(string, collapse = collapse))
     ans <- string[seq(1,(n-1))]
     if(n >= 2) {
-      remainder <- paste0(string[seq(n,length(string))],collapse=pattern)
+      remainder <- paste0(string[seq(n,length(string))],collapse=collapse)
       ans <- c(ans, remainder)  
     }
     ans
 }
 
 # replica str_split; to be replace if / when we take up stringr
-my_str_split <- function(string,pattern,n=3,fixed=FALSE) {
+my_str_split <- function(string,pattern,n=3,fixed=FALSE,collapse=pattern) {
   m <- strsplit(string, pattern, fixed = fixed)
-  lapply(m,collapsen,pattern=pattern,n=n)
+  lapply(m,collapsen,collapse=collapse,n=n)
 }
 
 ##' Create template data sets for simulation
@@ -589,5 +589,3 @@ mod_first <- function(cl) {
   msg <- sprintf("the first argument to %s must be a model object",fun)
   wstop(msg)
 }
-
-
