@@ -30,17 +30,31 @@
 static Rcpp::NumericMatrix OMEGADEF(1,1);
 static arma::mat OMGADEF(1,1,arma::fill::zeros);
 
-void dosimeta(void* prob_) {
+void dosimeta(void* prob_, int n) {
   odeproblem* prob = reinterpret_cast<odeproblem*>(prob_);
   arma::mat eta = prob->mv_omega(1);
+  if(n > int(eta.n_cols)) {
+    throw Rcpp::exception("simeta index out of bounds", false);
+  }
+  if(n > 0) {
+    prob->eta(n-1,eta(0,n-1));
+    return;
+  }
   for(unsigned int i=0; i < eta.n_cols; ++i) {
     prob->eta(i,eta(0,i)); 
   }
 }
 
-void dosimeps(void* prob_) {
+void dosimeps(void* prob_, int n) {
   odeproblem* prob = reinterpret_cast<odeproblem*>(prob_);
   arma::mat eps = prob->mv_sigma(1);
+  if(n > int(eps.n_cols)) {
+    throw Rcpp::exception("simeps index out of bounds", false);
+  }
+  if(n > 0) {
+    prob->eps(n-1,eps(0,n-1));
+    return;
+  }
   for(unsigned int i=0; i < eps.n_cols; ++i) {
     prob->eps(i,eps(0,i)); 
   }
