@@ -175,7 +175,6 @@ test_that("update arguments are passed", {
   expect_equal(mod@maxsteps, 5000)
 })
 
-
 test_that("no data generates error", {
   e <- ev()
   data <- as.data.frame(e)
@@ -196,3 +195,26 @@ test_that("negative istate is reported issue-457", {
   expect_true(grepl("consider increasing maxsteps", x[1]))
   expect_true(grepl("lsoda returned with negative istate: -1", x[3]))
 })
+
+test_that("simulate non-pred with negative times is allowed", {
+  mod <- house(delta=1)
+  dose <- ev(amt = 100, time = 4)
+  times <- seq(-10,10)
+  data <- expand_observations(dose, times)
+  
+  out1 <- mrgsim(mod, data)
+  expect_is(out1, "mrgsims")
+  expect_equal(data[["time"]],out1$time)
+  
+  out2 <- mrgsim(mod, events = dose, start = -10, end = 10, obsonly=TRUE)
+  expect_is(out2, "mrgsims")
+  expect_equal(times,out2$time)
+  
+  out3 <- filter_sims(out2, time >= 0)
+  out4 <- mrgsim(mod, events = dose, end = 10, obsonly=TRUE)
+  expect_is(out4, "mrgsims")
+  expect_equal(out3$DV,out4$DV)
+  
+  data$time[1] <- -8
+  expect_error(mrgsim(mod, data), "the data set is not sorted by time")
+})
